@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #include <unistd.h>
@@ -89,7 +89,7 @@ GST_START_TEST (test_no_clients)
 
   caps = gst_caps_from_string ("application/x-gst-check");
   buffer = gst_buffer_new_and_alloc (4);
-  gst_pad_set_caps (mysrcpad, caps);
+  gst_check_setup_events (mysrcpad, sink, caps, GST_FORMAT_BYTES);
   gst_caps_unref (caps);
   fail_unless (gst_pad_push (mysrcpad, buffer) == GST_FLOW_OK);
 
@@ -121,7 +121,7 @@ GST_START_TEST (test_add_client)
   ASSERT_CAPS_REFCOUNT (caps, "caps", 1);
   GST_DEBUG ("Created test caps %p %" GST_PTR_FORMAT, caps, caps);
   buffer = gst_buffer_new_and_alloc (4);
-  gst_pad_set_caps (mysrcpad, caps);
+  gst_check_setup_events (mysrcpad, sink, caps, GST_FORMAT_BYTES);
   ASSERT_CAPS_REFCOUNT (caps, "caps", 3);
   gst_buffer_fill (buffer, 0, "dead", 4);
   fail_unless (gst_pad_push (mysrcpad, buffer) == GST_FLOW_OK);
@@ -141,6 +141,19 @@ GST_START_TEST (test_add_client)
 
 GST_END_TEST;
 
+GST_START_TEST (test_add_client_in_null_state)
+{
+  GstElement *sink;
+
+  sink = setup_multifdsink ();
+
+  ASSERT_WARNING (g_signal_emit_by_name (sink, "add", 99));
+
+  cleanup_multifdsink (sink);
+}
+
+GST_END_TEST;
+
 #define fail_unless_read(msg,fd,size,ref) \
 G_STMT_START { \
   char data[size + 1]; \
@@ -260,7 +273,7 @@ GST_START_TEST (test_streamheader)
   gst_multifdsink_create_streamheader ("babe", "deadbeef", &hbuf1, &hbuf2,
       &caps);
   ASSERT_CAPS_REFCOUNT (caps, "caps", 1);
-  fail_unless (gst_pad_set_caps (mysrcpad, caps));
+  gst_check_setup_events (mysrcpad, sink, caps, GST_FORMAT_BYTES);
   /* one is ours, two from set_caps */
   ASSERT_CAPS_REFCOUNT (caps, "caps", 3);
 
@@ -352,7 +365,7 @@ GST_START_TEST (test_change_streamheader)
   gst_multifdsink_create_streamheader ("first", "header", &hbuf1, &hbuf2,
       &caps);
   ASSERT_CAPS_REFCOUNT (caps, "caps", 1);
-  fail_unless (gst_pad_set_caps (mysrcpad, caps));
+  gst_check_setup_events (mysrcpad, sink, caps, GST_FORMAT_BYTES);
   /* one is ours, two from set_caps */
   ASSERT_CAPS_REFCOUNT (caps, "caps", 3);
 
@@ -398,7 +411,7 @@ GST_START_TEST (test_change_streamheader)
 
   gst_multifdsink_create_streamheader ("second", "header", &hbuf1, &hbuf2,
       &caps);
-  fail_unless (gst_pad_set_caps (mysrcpad, caps));
+  gst_check_setup_events (mysrcpad, sink, caps, GST_FORMAT_BYTES);
   /* one to hold for the test and one to give away */
   ASSERT_BUFFER_REFCOUNT (hbuf1, "hbuf1", 2);
   ASSERT_BUFFER_REFCOUNT (hbuf2, "hbuf2", 2);
@@ -488,7 +501,7 @@ GST_START_TEST (test_burst_client_bytes)
   ASSERT_SET_STATE (sink, GST_STATE_PLAYING, GST_STATE_CHANGE_ASYNC);
 
   caps = gst_caps_from_string ("application/x-gst-check");
-  gst_pad_set_caps (mysrcpad, caps);
+  gst_check_setup_events (mysrcpad, sink, caps, GST_FORMAT_BYTES);
   GST_DEBUG ("Created test caps %p %" GST_PTR_FORMAT, caps, caps);
 
   /* push buffers in, 9 * 16 bytes = 144 bytes */
@@ -577,7 +590,7 @@ GST_START_TEST (test_burst_client_bytes_keyframe)
   ASSERT_SET_STATE (sink, GST_STATE_PLAYING, GST_STATE_CHANGE_ASYNC);
 
   caps = gst_caps_from_string ("application/x-gst-check");
-  gst_pad_set_caps (mysrcpad, caps);
+  gst_check_setup_events (mysrcpad, sink, caps, GST_FORMAT_BYTES);
   GST_DEBUG ("Created test caps %p %" GST_PTR_FORMAT, caps, caps);
 
   /* push buffers in, 9 * 16 bytes = 144 bytes */
@@ -668,7 +681,7 @@ GST_START_TEST (test_burst_client_bytes_with_keyframe)
   ASSERT_SET_STATE (sink, GST_STATE_PLAYING, GST_STATE_CHANGE_ASYNC);
 
   caps = gst_caps_from_string ("application/x-gst-check");
-  gst_pad_set_caps (mysrcpad, caps);
+  gst_check_setup_events (mysrcpad, sink, caps, GST_FORMAT_BYTES);
   GST_DEBUG ("Created test caps %p %" GST_PTR_FORMAT, caps, caps);
 
   /* push buffers in, 9 * 16 bytes = 144 bytes */
@@ -754,7 +767,7 @@ GST_START_TEST (test_client_next_keyframe)
   ASSERT_SET_STATE (sink, GST_STATE_PLAYING, GST_STATE_CHANGE_ASYNC);
 
   caps = gst_caps_from_string ("application/x-gst-check");
-  gst_pad_set_caps (mysrcpad, caps);
+  gst_check_setup_events (mysrcpad, sink, caps, GST_FORMAT_BYTES);
   GST_DEBUG ("Created test caps %p %" GST_PTR_FORMAT, caps, caps);
 
   /* now add our client */
@@ -800,6 +813,7 @@ multifdsink_suite (void)
   suite_add_tcase (s, tc_chain);
   tcase_add_test (tc_chain, test_no_clients);
   tcase_add_test (tc_chain, test_add_client);
+  tcase_add_test (tc_chain, test_add_client_in_null_state);
   tcase_add_test (tc_chain, test_streamheader);
   tcase_add_test (tc_chain, test_change_streamheader);
   tcase_add_test (tc_chain, test_burst_client_bytes);
