@@ -392,4 +392,19 @@ vorbisenc_suite (void)
   return s;
 }
 
-GST_CHECK_MAIN (vorbisenc);
+int
+main (int argc, char **argv)
+{
+  int nf;
+
+  Suite *s = vorbisenc_suite ();
+  SRunner *sr = srunner_create (s);
+
+  gst_check_init (&argc, &argv);
+
+  srunner_run_all (sr, CK_NORMAL);
+  nf = srunner_ntests_failed (sr);
+  srunner_free (sr);
+
+  return nf;
+}
